;----------------------------------------------------------------------------

;Title	: DSLR IR remote controller
;         supports:
;         Nikon D40/D50/D70/D70s/D80
;         Canon EOS 300D/350D/400D
;         Pentax *ist D/Ds/Dl/Ds2
;Version: 1.00
;Target	: ATtiny12
;Author	: wubblick@yahoo.com

;AVR Assembler 2.0 Beta

;Fuse bits:

;Int RCosc, Startup 4.2ms + 6 CK
;No BOD function

;Ports usage:
;Port PB1 (INT0) used for push button, active LOW.
;Ports PB0, PB2, PB3, PB4 used in parallel (via current
;limitting resistors) to drive IR LED, active LOW.

;Generated IR pattern (Nikon):

;Pulse 2000 uS -> Pause 27830 uS -> Pulse 390 uS -> Pause 1580 uS ->
;Pulse 410 uS -> Pause 3580 uS -> Pulse 400 uS -> Pause 63200 uS ->
;Pulse 2000 uS -> Pause 27830 uS -> Pulse 390 uS -> Pause 1580 uS ->
;Pulse 410 uS -> Pause 3580 uS -> Pulse 400 uS

;All pulses are modulated with 38.4 kHz

;Generated IR pattern (Canon):

;Pulse 480 uS -> Pause 7324 uS -> Pulse 480 uS

;All pulses are modulated with 33.0 kHz

;Generated IR pattern (Pentax):

;Pulse 13000 uS -> Pause 3000 uS ->
;Pulse 1000 uS -> Pause 1000 uS ->  (1)
;Pulse 1000 uS -> Pause 1000 uS ->  (2)
;                                   (...)
;Pulse 1000 uS                      (7)

;All pulses are modulated with 38.0 kHz

;----------------------------------------------------------------------------

.include "tn12def.inc"

;----------------------------------------------------------------------------

#define Nikon				;uncomment to select Nikon
;#define Canon				;uncomment to select Canon
;#define Pentax				;uncomment to select Pentax

;----------------------------------------------------------------------------

;Constantes:

.equ	FCLK = 1200000			;clock frequency (internal RC) [Hz]

#ifdef Nikon
.equ	FMOD = 38400			;modulation frequency [Hz]
#endif

#ifdef Canon
.equ	FMOD = 33000			;modulation frequency [Hz]
#endif

#ifdef Pentax
.equ	FMOD = 38000			;modulation frequency [Hz]
#endif

;----------------------------------------------------------------------------

;Ports definition:

.equ	LED1	= exp2(PB0)		;IR LED port 1
.equ	PBTN	= exp2(PB1)		;push button port
.equ	LED2	= exp2(PB2)		;IR LED port 2
.equ	LED3	= exp2(PB3)		;IR LED port 3
.equ	LED4	= exp2(PB4)		;IR LED port 4

.equ	LED	= LED1 | LED2 | LED3 | LED4

.equ	DIRB	= LED			;port B direction
.equ	PUPB	= PBTN | LED		;port B pull-ups/initial values

.equ	LED_ON_MASK  = PBTN & ~LED	;mask to switsh LED on
.equ	LED_OFF_MASK = PBTN | LED	;mask to switsh LED off
 
;----------------------------------------------------------------------------

;Macro: generate delay @0 [Cycles]:

.macro	Delay
.if (@0 % 3) > 1
	nop
.endif
.if (@0 % 3) > 0
	nop
.endif
	ldi	Cnt,int(@0 / 3)		;1
loop:	dec	Cnt			;1
	brne	loop			;2(1)
.endm

;----------------------------------------------------------------------------

;Macro: generate modulated with FMOD [Hz] IR pulse @0 [uS]:

.equ	MCYC  = FCLK / FMOD		;modulation period [cycles]
.equ	MCYC1 = MCYC / 2		;LED on duration [cycles]
.equ	MCYC0 = MCYC - MCYC1		;LED off duration [cycles]

.macro	Pulse
	ldi	tempL,byte1((@0 * FMOD / 100000 - 5) / 10)
	ldi	tempH,byte2((@0 * FMOD / 100000 - 5) / 10)

loop:	ldi	temp,LED_ON_MASK	;1
	out	PORTB,temp		;1
	Delay(MCYC1 - 2)		;MCYC1 - 2

	ldi	temp,LED_OFF_MASK	;1
	out	PORTB,temp		;1
	Delay(MCYC0 - 6)		;MCYC0 - 6
	subi	tempL,1			;1
	sbci	tempH,0			;1
	brcc	loop			;2(1)
.endm

;----------------------------------------------------------------------------

;Macro: generate pause @0 [uS]:

.macro	Pause
	ldi	tempL,byte1((@0 * FCLK / 20 / 100000 - 5) / 10)
	ldi	tempH,byte2((@0 * FCLK / 20 / 100000 - 5) / 10)
loop:	Delay(16)			;16
	subi	tempL,1			;1
	sbci	tempH,0			;1
	brcc	loop			;2(1)
.endm

;----------------------------------------------------------------------------

;Global Register Variables:

.def	temp	= r16			;temporary register temp
.def	tempL	= r17			;temporary register tempL
.def	tempH	= r18			;temporary register tempH
.def	Cnt	= r19			;temporary register Cnt

;----------------------------------------------------------------------------

.CSEG	;Code segment

;----------------------------------------------------------------------------

;Interrupt vectors:

.org	0
	rjmp	Init			;reset vector
.org	INT0addr
	reti				;push button interrupt service

;----------------------------------------------------------------------------

OscCV:	.dw 0x21			;internal RC oscillator calibration
					;value. Copy here value from
					;high byte of address 0x0000
					;in the signature address space.

;----------------------------------------------------------------------------

;Initialization:

Init:	ldi	ZL,byte1(OscCV*2)	;read oscillator calibration value
	ldi	ZH,byte2(OscCV*2)
	lpm
	out	OSCCAL,r0		;calibrate oscillator

	ldi	temp,(1<<ACD)		;analog comparator disable
	out	ACSR,temp

	ldi	temp,PUPB
	out	PORTB,temp		;init PORTB and on/off pullup
	ldi	temp,DIRB	
	out	DDRB,temp		;set PORTB direction

	ldi	temp,(1<<SE) | (1<<SM)	;sleep - power down, level INT0
	out	MCUCR,temp
	ldi	temp,(1<<INT0)		;INT0 enable
	out	GIMSK,temp

;----------------------------------------------------------------------------

;Main program:

Main:	sei				;global interupt enable
	sleep				;go to power down
	cli				;global interupt disable
	
#ifdef Nikon
	Pulse(2000)			;generate pattern
	Pause(27830)
	Pulse(390)
	Pause(1580)
	Pulse(410)
	Pause(3580)
	Pulse(400)

	Pause(63200)

	Pulse(2000)
	Pause(27830)
	Pulse(390)
	Pause(1580)
	Pulse(410)
	Pause(3580)
	Pulse(400)
#endif

#ifdef Canon
	Pulse(480)			;generate pattern
	Pause(7324)
	Pulse(480)
#endif

#ifdef Pentax
	Pulse(13000)			;generate pattern
	Pause(3000)
	Pulse(1000)
	Pause(1000)
	Pulse(1000)
	Pause(1000)
	Pulse(1000)
	Pause(1000)
	Pulse(1000)
	Pause(1000)
	Pulse(1000)
	Pause(1000)
	Pulse(1000)
	Pause(1000)
	Pulse(1000)
	Pause(1000)
#endif

wait1:	sbis	PINB,log2(PBTN)		;wait push button release
	rjmp	wait1
	ldi	temp,100		;load debounce counter
wait0:	sbis	PINB,log2(PBTN)		;check push button state
	rjmp	wait1			;restart debounce if changed
	Pause(1000)			;wait 1 mS
	dec	temp			;advance counter
	brne	wait0

	rjmp	Main

;----------------------------------------------------------------------------
